@@ -1,58 +1,5 @@
 /* ccchh_dooris_desktop_widget.c */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <stdlib.h>
-#include <errno.h>
-#include <unistd.h>
-#include <string.h>
-
-#include <glib.h>
-#include <glib/gi18n-lib.h> /* Declares _(). */
-#include <gtk/gtk.h>
-
-#include <hildon/hildon.h>
-#include <libhildondesktop/libhildondesktop.h>
-#include <conic.h> /* libconic - Internet Connectivity library */
-
-#include <libgnomevfs/gnome-vfs.h>
-
-#include "ccchh-dooris-widget.h"
-
-// max download size
-#define MAX_HTTP_SIZE 1024
-
-// minimum/maximum selectable update interval
-// in minutes
-#define UPDATE_INTERVAL_MIN	5
-#define UPDATE_INTERVAL_MAX	1440
-
-// paths to icon files
-#define IMAGE_DOORIS_OPEN			DATADIR"/dooris_open.png"
-#define IMAGE_DOORIS_CLOSED			DATADIR"/dooris_closed.png"
-#define IMAGE_DOORIS_FNORD			DATADIR"/dooris_fnord.png"
-#define IMAGE_OVERLAY_DISCONNECTED	DATADIR"/dooris_notavail.png"
-
-// widget size
-#define IMAGE_WIDGET_WIDTH	102
-#define IMAGE_WIDGET_HEIGHT	72
-
-// transparency
-#define IMAGE_ALPHA_NORMAL		1.0
-#define IMAGE_ALPHA_HIGHLIGHT	0.65
-
-#define _DEBUG
-#ifdef _DEBUG
-// redefine g_debug() to g_message()
-// TODO: find out how to get debug logs from hildon-desktop
-#define g_debug g_message
-#define USER_AGENT_VERSION_STRING (debug version)
-#else
-#define USER_AGENT_VERSION_STRING (some version number)
-#endif
-
 /**
  * CCCHH DoorIs Fnord Desktop Widget
  *
@@ -80,6 +27,59 @@
  *
  */
 
+
+// TODO:
+// - sprinkle with type-checking macros and g_assert
+// - save settings in gconf
+// - handle button_press, button_release, leave_notify
+//   - highlight on button press, perform action on release
+
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <stdlib.h>
+#include <errno.h>
+#include <unistd.h>
+#include <string.h>
+
+#include <glib.h>
+#include <glib/gi18n-lib.h> /* Declares _(). */
+
+#include <gtk/gtk.h>
+
+#include <hildon/hildon.h>
+#include <libhildondesktop/libhildondesktop.h>
+
+#include <conic.h> /* libconic - Internet Connectivity library */
+
+#include <libgnomevfs/gnome-vfs.h>
+
+#include "ccchh-dooris-widget.h"
+
+
+// #define DEBUG
+
+#ifdef DEBUG
+
+// redefine g_debug() to g_message()
+// TODO: find out how to get debug logs from hildon-desktop
+#define g_debug g_message
+
+#define USER_AGENT_VERSION_STRING (debug version)
+
+#else
+
+// check g_debug statements for syntax errors but don't emit code
+// (compiler should optimize it)
+#define g_debug(fmt...) do { g_message(fmt); } while (0)
+
+#define USER_AGENT_VERSION_STRING (some version number)
+
+#endif // DEBUG
+
+
 /** This is like G_DEFINE_TYPE(), but it also 
  * implements standard functions such as hd_plugin_module_load(), 
  * which hildon-desktop expects to be able to call on this library.
@@ -93,6 +93,7 @@ HD_DEFINE_PLUGIN_MODULE (CCCHHDoorIsDesktopWidget, ccchh_dooris_desktop_widget,
  
 // Code to be reworked:
 
+/*
 // TODO: is this really the right thing to do?
 // code copied from the internets
 // maybe use gtk-timers instead of "for" loop
@@ -113,95 +114,152 @@ static void gtk_sleep (unsigned int ms)
       gtk_main_iteration ();
   }
 }
-
-
-// TODO: better implementation.
-// maybe use glibcurl (http://atterer.net/glibcurl/)
-// or GnomeVFS
-static dooris_state_t
-fetch_current_state (CCCHHDoorIsDesktopWidget *self)
-{
-	static GStaticMutex local_fetch_mutex = G_STATIC_MUTEX_INIT;
-
-	gulong ts;
-	dooris_state_t state;
-	char *line, *tokctx;
-
-	GnomeVFSHandle *gvfs_handle;
-	GnomeVFSResult gvfs_errno;
-	GnomeVFSFileSize bytes_read;
-
-	char buffer[MAX_HTTP_SIZE+1];
-	memset(buffer, 0, sizeof(buffer));
-
-	g_debug("ccchh-dooris-widget: fetch_current_state()");
-
-	if (!g_static_mutex_trylock(&local_fetch_mutex)) {
-		g_warning("ccchh-dooris-widget: BUG: local fetch() lock set, not updating");
-		return DOORIS_FNORD;
-	}
-
-	g_debug("ccchh-dooris-widget: fetching state from URL: %s", self->url);
-
-// why bother? we check error on open
-/*
-GnomeVFSURI *uri_struct = gnome_vfs_uri_new(self->url);
-if (!gnome_vfs_uri_exists(uri_struct)) {
-	g_warning("ccchh-dooris-widget: URL does not exist");
-	g_static_mutex_unlock(&local_fetch_mutex);
-	return DOORIS_FNORD;
-}
-gnome_vfs_uri_unref(uri_struct);
 */
 
-	g_debug("ccchh-dooris-widget: gnome_vfs_open()");
-	gvfs_errno = gnome_vfs_open(&gvfs_handle, self->url, GNOME_VFS_OPEN_READ);
 
-	if (gvfs_errno != GNOME_VFS_OK) {
-		g_warning("ccchh-dooris-widget: gnome_vfs_open: %s (%d)\n", gnome_vfs_result_to_string(gvfs_errno), gvfs_errno);
-		g_static_mutex_unlock(&local_fetch_mutex);
-		return DOORIS_FNORD;
+static inline gboolean
+fetch_update_lock (CCCHHDoorIsDesktopWidget *self)
+{
+	if (!g_static_mutex_trylock(&(self->fetch_lock))) {
+		return FALSE;
 	}
 
-	g_debug("ccchh-dooris-widget: gnome_vfs_read()");
-	gvfs_errno = gnome_vfs_read(gvfs_handle, buffer, MAX_HTTP_SIZE, &bytes_read);
+	// acknowledge button, make icon transparent
+	// TODO: not really highlighting the button
+	// also it doesn't always redraw in time
+	self->current_alpha = IMAGE_ALPHA_HIGHLIGHT;
+	force_redraw(self);
 
-	if (gvfs_errno != GNOME_VFS_OK) {
-		g_warning("ccchh-dooris-widget: gnome_vfs_read: %s (%d)\n", gnome_vfs_result_to_string(gvfs_errno), gvfs_errno);
-		g_static_mutex_unlock(&local_fetch_mutex);
-		return DOORIS_FNORD;
+	return TRUE;
+}
+
+
+static inline void
+fetch_update_unlock (CCCHHDoorIsDesktopWidget *self)
+{
+	self->current_alpha = IMAGE_ALPHA_NORMAL;
+	force_redraw(self);
+
+	g_static_mutex_unlock(&(self->fetch_lock));
+}
+
+
+// initialize async i/o
+// callback: fetch_update_async_open_cb()
+static void
+fetch_update_start (CCCHHDoorIsDesktopWidget *self)
+{
+	g_debug("ccchh-dooris-widget: fetch_update_start()");
+
+	if (!self->connected) {
+		g_debug("ccchh-dooris-widget: not connected, not fetching new state");
+		return;
 	}
 
-	g_debug("ccchh-dooris-widget: read %lld bytes", bytes_read);
-// rest of buffer already zeroed by memset()
-//	buffer[bytes_read] = '\0';
+	if (!fetch_update_lock(self)) {
+		g_message("ccchh-dooris-widget: fetch_lock set, not updating");
+		return;
+	}
 
-	g_debug("ccchh-dooris-widget: gnome_vfs_close()");
-	gnome_vfs_close(gvfs_handle);
- 
-	self->last_change_ts = 0L;
-	self->last_report_ts = 0L;
+	g_debug("ccchh-dooris-widget: begin update from URL: %s", self->url);
+
+
+/*
+	if (self->gvfs_handle) {
+		g_object_unref(self->gvfs_handle);
+		self->gvfs_handle = NULL;
+	}
+*/
+	// TODO: set timeout to cancel async open
+
+	gnome_vfs_async_open(&self->gvfs_handle, self->url, GNOME_VFS_OPEN_READ,
+		GNOME_VFS_PRIORITY_MIN, fetch_update_async_open_cb, self);
+}
+
+
+static void
+fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
+    gpointer data)
+{
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+
+	g_debug("ccchh-dooris-widget: fetch_update_async_open_cb()");
+
+	if (result != GNOME_VFS_OK) {
+		g_warning("ccchh-dooris-widget: error opening URL: %s", gnome_vfs_result_to_string (result));
+		fetch_update_unlock(self);
+		return;
+	}
+
+	if (self->gvfs_input_buffer) {
+		g_free(self->gvfs_input_buffer);
+		self->gvfs_input_buffer = NULL;
+	}
+
+	if (!(self->gvfs_input_buffer = g_try_malloc0(MAX_HTTP_SIZE+1))) {
+		g_warning("ccchh-dooris-widget: failed to allocate input buffer");
+		fetch_update_unlock(self);
+		return;
+	}
+
+	// TODO: set timeout to cancel async read
+
+	gnome_vfs_async_read (handle, self->gvfs_input_buffer, MAX_HTTP_SIZE,
+		fetch_update_async_read_cb, self);
+}
+
+
+static void
+fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
+	gpointer buffer, GnomeVFSFileSize bytes_requested, GnomeVFSFileSize bytes_read,
+	gpointer data)
+{
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+
+	g_debug("ccchh-dooris-widget: fetch_update_async_read_cb()");
+
+	if (result != GNOME_VFS_OK) {
+		g_warning("ccchh-dooris-widget: read error: %s", gnome_vfs_result_to_string (result));
+		fetch_update_unlock(self);
+		return;
+	}
+
+	g_debug("ccchh-dooris-widget: read %"GNOME_VFS_SIZE_FORMAT_STR" bytes", bytes_read);
+
+	// TODO: set timeout to cancel close
+
+	// close file
+	gnome_vfs_async_close (handle, fetch_update_async_close_cb, self);
+
+	// meanwhile, parse input buffer
+
+	gulong ts, new_change_ts, new_report_ts;
+	dooris_state_t new_state;
+	char *line, *tokctx;
+
+	new_state = DOORIS_FNORD;
+	new_change_ts = 0L;
+	new_report_ts = 0L;
 	tokctx = NULL;
 
 	// first line of body is either "locked" or "unlocked"
 
-	if (!(line = strtok_r(buffer, "\n", &tokctx))) {
+	if (!(line = strtok_r(self->gvfs_input_buffer, "\n", &tokctx))) {
 		g_warning("ccchh-dooris-widget: first line of response not found");
-		g_static_mutex_unlock(&local_fetch_mutex);
-		return DOORIS_FNORD;
+		process_update(self, new_state, new_change_ts, new_report_ts);
+		return;
 	}
 
 	if (!strcmp(line, "locked")) {
-		state = DOORIS_CLOSED;
+		new_state = DOORIS_CLOSED;
 		g_debug("ccchh-dooris-widget: current state: closed");
 	} else if (!strcmp(line, "unlocked")) {
-		state = DOORIS_OPEN;
+		new_state = DOORIS_OPEN;
 		g_debug("ccchh-dooris-widget: current state: open");
-	}
-	else {
+	} else {
 		g_warning("ccchh-dooris-widget: could not parse response");
-		g_static_mutex_unlock(&local_fetch_mutex);
-		return DOORIS_FNORD;
+		process_update(self, new_state, new_change_ts, new_report_ts);
+		return;
 	}
 
 	// second line of body is last sensor report timestamp
@@ -210,9 +268,11 @@ gnome_vfs_uri_unref(uri_struct);
 		errno = 0;
 		ts = strtoul(line, (char **) NULL, 10);
 		if (ts && !errno) {
-			self->last_report_ts = ts;
+			new_report_ts = ts;
+			g_debug("ccchh-dooris-widget: last_report_ts: %lu, errno: %i\n", ts, errno);
+		} else {
+			g_warning("ccchh-dooris-widget: could not parse last_report_ts");
 		}
-		g_debug("ccchh-dooris-widget: last_report_ts: %lu, errno: %i\n", ts, errno);
 
 		// third line of body is last state change timestamp
 
@@ -220,17 +280,44 @@ gnome_vfs_uri_unref(uri_struct);
 			errno = 0;
 			ts = strtoul(line, (char **) NULL, 10);
 			if (ts && !errno) {
-				self->last_change_ts = ts;
+				new_change_ts = ts;
+				g_debug("ccchh-dooris-widget: last_change_ts: %lu, errno: %i\n", ts, errno);
+			} else {
+				g_warning("ccchh-dooris-widget: could not parse last_change_ts");
 			}
-			g_debug("ccchh-dooris-widget: last_change_ts: %lu, errno: %i\n", ts, errno);
 		}
 	}
 
-	g_static_mutex_unlock(&local_fetch_mutex);
+	// TODO: reuse small buffers
+	g_free(self->gvfs_input_buffer);
+	self->gvfs_input_buffer = NULL;
 
-	return state;
+	process_update(self, new_state, new_change_ts, new_report_ts);
 }
 
+
+static void
+fetch_update_async_close_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
+	gpointer data)
+{
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+
+	g_debug("ccchh-dooris-widget: fetch_update_async_close_cb()");
+
+	if (result != GNOME_VFS_OK) {
+		g_debug("ccchh-dooris-widget: error on close: %s", gnome_vfs_result_to_string (result));
+		// TODO: what now?
+	}
+
+// TODO: think again
+//	g_usleep(100000);
+
+	fetch_update_unlock(self);
+}
+
+
+
+
 // stop update timer
 // returns TRUE if timer was stopped
 // returns FALSE if no timer was running
@@ -247,6 +334,7 @@ stop_update_timer(CCCHHDoorIsDesktopWidget *self)
 	return FALSE;
 }
 
+
 // start update timer
 // returns TRUE if timer was started
 // returns FALSE if timer was already running
@@ -269,6 +357,7 @@ start_update_timer(CCCHHDoorIsDesktopWidget *self)
 	return FALSE;
 }
 
+
 // install libconic connection event handler
 // returns TRUE if handler was installed
 // returns FALSE if handler was already set
@@ -291,6 +380,7 @@ install_connection_handler(CCCHHDoorIsDesktopWidget *self)
 	return FALSE;
 }
 
+
 // remove libconic connection event handler
 // returns TRUE if handler was removed
 // returns FALSE if no handler was set
@@ -308,37 +398,38 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 	return FALSE;
 }
 
+
+
+
 // update internal state
-// returns TRUE on significant state change
-// (i.e. state is valid and has changed since last valid state)
-// TODO: glib locking?
-// this function should not be called simultaneously
-// timer and button events could call update() at the same time
-static gboolean
-update (CCCHHDoorIsDesktopWidget *self)
+
+static void
+process_update (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
+	gulong new_change_ts, gulong new_report_ts)
 {
-	static GStaticMutex local_update_mutex = G_STATIC_MUTEX_INIT;
+	static GStaticMutex process_lock = G_STATIC_MUTEX_INIT;
 
-	g_debug("ccchh-dooris-widget: update()");
+	g_debug("ccchh-dooris-widget: process_update()");
 
-	if (!g_static_mutex_trylock(&local_update_mutex)) {
-		g_warning("ccchh-dooris-widget: BUG: local update() lock set, not updating");
-		return FALSE;
+	if (!g_static_mutex_trylock(&process_lock)) {
+		g_warning("ccchh-dooris-widget: BUG: process_update() lock set, not updating");
+		return;
 	}
 
-	if (!self->connected) {
-		g_debug("ccchh-dooris-widget: not connected, not fetching new state");
-		g_static_mutex_unlock(&local_update_mutex);
-		return FALSE;
-	}
+	self->current_state = new_state;
+	self->last_change_ts = new_change_ts;
+	self->last_report_ts = new_report_ts;
 
-	// retrieve state from server
-	self->current_state = fetch_current_state(self);
-
-	if (self->current_state == self->last_state) {
+	if (new_state == self->last_state) {
 		g_debug("ccchh-dooris-widget: state unchanged");
-		g_static_mutex_unlock(&local_update_mutex);
-		return FALSE;
+
+		if (self->force_banner) {
+			self->force_banner = FALSE;
+			show_status_banner(self);
+		}
+
+		g_static_mutex_unlock(&process_lock);
+		return;
 	}
 
 	// release old icon surface
@@ -367,15 +458,25 @@ update (CCCHHDoorIsDesktopWidget *self)
 
 	// state is valid and has changed since last known state
 	if (self->current_state != DOORIS_FNORD && self->current_state != self->last_known_state) {
+
 		self->last_known_state = self->current_state;
 		g_debug("ccchh-dooris-widget: state changed");
-		g_static_mutex_unlock(&local_update_mutex);
-		return TRUE;
+
+		// state changed: redraw icon
+		force_redraw(self);
+
+		// flash banner if notifications are enabled
+		if (self->notify) {
+			self->force_banner = TRUE;
+		}
 	}
 
-	g_static_mutex_unlock(&local_update_mutex);
+	if (self->force_banner) {
+		self->force_banner = FALSE;
+		show_status_banner(self);
+	}
 
-	return FALSE;
+	g_static_mutex_unlock(&process_lock);
 }
 
 
@@ -507,28 +608,7 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 		return FALSE;
 	}
 
-	// update() must be called only once at a time
-	if (!g_static_mutex_trylock(&(self->update_mutex))) {
-		g_debug("ccchh-dooris-widget: update lock set, not updating");
-		return TRUE;
-	}
-
-	// update internal state
-	if (!update(self)) {
-		g_static_mutex_unlock(&(self->update_mutex));
-		// nothing significant to report, keep the timer running
-		// TODO: this could miss to redraw a change to/from DOORIS_FNORD
-		return TRUE;
-	}
-
-	// state changed: redraw icon
-	force_redraw(self);
-
-	// flash banner if notifications are enabled
-	if (self->notify)
-		show_status_banner(self);
-
-	g_static_mutex_unlock(&(self->update_mutex));
+	fetch_update_start(self);
 
 	return TRUE; /* keep running this event */
 }
@@ -537,20 +617,19 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 static gboolean
 ccchh_dooris_desktop_widget_expose_event (GtkWidget *widget, GdkEventExpose *event)
 {    
-	cairo_t *cr  = gdk_cairo_create (widget->window);
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (widget);
 
 	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_expose_event()");
 
 	if (!self->image_current) {
 		g_debug("ccchh-dooris-widget: expose event without image");
-		// no state yet, call update()
-		// flash banner if necessary and enabled
-		if (update(self) && self->notify) {
-			show_status_banner(self);
-		}
+		self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_FNORD);
+		// no state yet, begin update
+		fetch_update_start(self);
 	}
 
+	cairo_t *cr  = gdk_cairo_create (widget->window);
+
 	/* Clip only the exposed area of the cairo context,  
 	 * to potentially avoid unnecessary drawing:
 	 */
@@ -577,17 +656,11 @@ ccchh_dooris_desktop_widget_on_button_press_event (GtkWidget *widget,
 	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_on_button_press_event()");
  
 	// update() must be called only once at a time
-	if (!g_static_mutex_trylock(&(self->update_mutex))) {
-		g_debug("ccchh-dooris-widget: update lock set, not updating");
+	if (!g_static_mutex_trylock(&(self->button_lock))) {
+		g_debug("ccchh-dooris-widget: button_lock set, not updating");
 		return TRUE;
 	}
 
-	// acknowledge button, make icon transparent
-	// TODO: not really highlighting the button
-	// also it doesn't always redraw in time
-	self->current_alpha = IMAGE_ALPHA_HIGHLIGHT;
-	force_redraw(self);
-
 	// request connection from libconic, if offline and
 	// force_connection is enabled
 	// TODO: this does not work immediately, the call to update()
@@ -601,36 +674,15 @@ ccchh_dooris_desktop_widget_on_button_press_event (GtkWidget *widget,
 			g_warning("ccchh-dooris-widget: request for connection failed");
 	}
 
-	// TODO: think again
-	// we want the button to stay transparent for a few ms,
-	// even if update() returns fast
-	// also, running the main loop from within a button event
-	// could call the function again. re-ent-what?
-	gtk_sleep(100);
-
-	// update internal state, redraw on change
-	// TODO: this could miss to redraw a change to/from DOORIS_FNORD
-	// although the next force_redraw() catches it
-	if (update(self)) {
-		force_redraw(self);
-	}
-
-	// always display the banner when the button is pressed
-	show_status_banner(self);
-
-	// redraw the icon opaque
-	self->current_alpha = IMAGE_ALPHA_NORMAL;
-	force_redraw(self);
-
-	// TODO: think again, see above
-	gtk_sleep(100);
+	self->force_banner = TRUE;
+	fetch_update_start(self);
 
 	// if a timer was running, reset it
 	if (stop_update_timer(self)) {
 		start_update_timer(self);
 	}
 
-	g_static_mutex_unlock(&(self->update_mutex));
+	g_static_mutex_unlock(&(self->button_lock));
 
 	return TRUE; /* Prevent further handling of this signal. */
 }
@@ -750,12 +802,10 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 
 		// status URL
 		if (self->url) {
-			// is there any difference between free() and g_free()?
-//			g_free(self->url);
 			free(self->url);
 			self->url = NULL;
 		}
-//		self->url = g_strdup(gtk_entry_get_text(GTK_ENTRY (self->url_entry)));
+
 		// try to build a valid URI from user input
 		self->url = gnome_vfs_make_uri_from_input(gtk_entry_get_text(GTK_ENTRY (self->url_entry)));
 		g_debug("ccchh-dooris-widget: new URL: %s\n", self->url);
@@ -964,7 +1014,7 @@ ccchh_dooris_desktop_widget_realize (GtkWidget *widget)
 }
 
 
-// TODO: this function gets called twice. why?
+// TODO: this function is called twice. why?
 static void
 ccchh_dooris_desktop_widget_dispose (GObject *object)
 {
@@ -1009,6 +1059,22 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 		self->url = NULL;
 	}
 
+	g_static_mutex_free(&(self->button_lock));
+	g_static_mutex_free(&(self->fetch_lock));
+
+/*
+	if (self->gvfs_handle) {
+		g_object_unref(self->gvfs_handle);
+		self->gvfs_handle = NULL;
+	}
+*/
+
+	if (self->gvfs_input_buffer) {
+		g_free(self->gvfs_input_buffer);
+		self->gvfs_input_buffer = NULL;
+	}
+
+
 	/* Call the base class's implementation: */
 	G_OBJECT_CLASS (ccchh_dooris_desktop_widget_parent_class)->finalize (object);
 }
@@ -1043,12 +1109,14 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 {
 	g_debug("ccchh-dooris-widget: ccchh_dooris_desktop_widget_init()");
 
+/*
 	// initialize GnomeVFS library
 	if (!gnome_vfs_init()) {
 		// FIXME: crashes the whole desktop process
 		g_error("ccchh-dooris-widget: could not initialize GnomeVFS library");
 		abort();
 	}
+*/
  
 	// default settings
 	self->notify = TRUE;
@@ -1076,10 +1144,16 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	self->image_current = NULL;
 	self->current_alpha = IMAGE_ALPHA_NORMAL;
 
-	g_static_mutex_init(&(self->update_mutex));
+	g_static_mutex_init(&(self->button_lock));
+	g_static_mutex_init(&(self->fetch_lock));
+
 	self->force_banner = FALSE;
+
 	self->disconnecting = NULL;
 
+	self->gvfs_handle = NULL;
+	self->gvfs_input_buffer = NULL;
+
 	// set widget size
 	// TODO: is this the right way?
 	gtk_widget_set_size_request (GTK_WIDGET (self), IMAGE_WIDGET_WIDTH, IMAGE_WIDGET_HEIGHT);
